@@ -7,6 +7,7 @@
 #include "base/object.hpp"
 #include <boost/range/iterator.hpp>
 #include <boost/utility/string_view.hpp>
+#include <functional>
 #include <string>
 #include <iosfwd>
 
@@ -178,6 +179,12 @@ String::ConstIterator range_end(const String& x);
 
 }
 
+template<>
+struct std::hash<icinga::String>
+{
+	std::size_t operator()(const icinga::String& s) const noexcept;
+};
+
 extern template class std::vector<icinga::String>;
 
 namespace boost
