@@ -8,12 +8,6 @@
 
 using namespace icinga;
 
-static void InvokeAttributeHandlerHelper(const Function::Ptr& callback,
-	const Object::Ptr& object, const Value& cookie)
-{
-	callback->Invoke({ object });
-}
-
 static void TypeRegisterAttributeHandler(const String& fieldName, const Function::Ptr& callback)
 {
 	ScriptFrame *vframe = ScriptFrame::GetCurrentFrame();
@@ -21,7 +15,9 @@ static void TypeRegisterAttributeHandler(const String& fieldName, const Function
 	REQUIRE_NOT_NULL(self);
 
 	int fid = self->GetFieldId(fieldName);
-	self->RegisterAttributeHandler(fid, std::bind(&InvokeAttributeHandlerHelper, callback, _1, _2));
+	self->RegisterAttributeHandler(fid, [callback](const Object::Ptr& object, const Value& cookie) {
+		callback->Invoke({ object });
+	});
 }
 
 Object::Ptr TypeType::GetPrototype()
