@@ -3,24 +3,20 @@
 #include "config/applyrule.hpp"
 #include "base/logger.hpp"
 #include <set>
+#include <unordered_set>
 
 using namespace icinga;
 
 ApplyRule::RuleMap ApplyRule::m_Rules;
 ApplyRule::TypeMap ApplyRule::m_Types;
 
-ApplyRule::ApplyRule(String targetType, String name, Expression::Ptr expression,
+ApplyRule::ApplyRule(String name, Expression::Ptr expression,
 	Expression::Ptr filter, String package, String fkvar, String fvvar, Expression::Ptr fterm,
 	bool ignoreOnError, DebugInfo di, Dictionary::Ptr scope)
-	: m_TargetType(std::move(targetType)), m_Name(std::move(name)), m_Expression(std::move(expression)), m_Filter(std::move(filter)), m_Package(std::move(package)), m_FKVar(std::move(fkvar)),
+	: m_Name(std::move(name)), m_Expression(std::move(expression)), m_Filter(std::move(filter)), m_Package(std::move(package)), m_FKVar(std::move(fkvar)),
 	m_FVVar(std::move(fvvar)), m_FTerm(std::move(fterm)), m_IgnoreOnError(ignoreOnError), m_DebugInfo(std::move(di)), m_Scope(std::move(scope)), m_HasMatches(false)
 { }
 
-String ApplyRule::GetTargetType() const
-{
-	return m_TargetType;
-}
-
 String ApplyRule::GetName() const
 {
 	return m_Name;
@@ -75,7 +71,22 @@ void ApplyRule::AddRule(const String& sourceType, const String& targetType, cons
 	const Expression::Ptr& expression, const Expression::Ptr& filter, const String& package, const String& fkvar,
 	const String& fvvar, const Expression::Ptr& fterm, bool ignoreOnError, const DebugInfo& di, const Dictionary::Ptr& scope)
 {
-	m_Rules[sourceType].push_back(ApplyRule(targetType, name, expression, filter, package, fkvar, fvvar, fterm, ignoreOnError, di, scope));
+	auto actualTargetType (&targetType);
+
+	if (*actualTargetType == "") {
+		auto& targetTypes (GetTargetTypes(sourceType));
+
+		if (targetTypes.size() == 1u) {
+			actualTargetType = &targetTypes[0];
+		}
+	}
+
+	ApplyRule::Ptr rule = new ApplyRule(name, expression, filter, package, fkvar, fvvar, fterm, ignoreOnError, di, scope);
+	auto& rules (m_Rules[Type::GetByName(sourceType).get()]);
+
+	if (!AddTargetedRule(rule, *actualTargetType, rules)) {
+		rules.Regular[Type::GetByName(*actualTargetType).get()].emplace_back(std::move(rule));
+	}
 }
 
 bool ApplyRule::EvaluateFilter(ScriptFrame& frame) const
@@ -111,12 +122,14 @@ bool ApplyRule::IsValidTargetType(const String& sourceType, const String& target
 	return false;
 }
 
-std::vector<String> ApplyRule::GetTargetTypes(const String& sourceType)
+const std::vector<String>& ApplyRule::GetTargetTypes(const String& sourceType)
 {
 	auto it = m_Types.find(sourceType);
 
-	if (it == m_Types.end())
-		return std::vector<String>();
+	if (it == m_Types.end()) {
+		static const std::vector<String> noTypes;
+		return noTypes;
+	}
 
 	return it->second;
 }
@@ -131,23 +144,56 @@ bool ApplyRule::HasMatches() const
 	return m_HasMatches;
 }
 
-std::vector<ApplyRule>& ApplyRule::GetRules(const String& type)
+const std::vector<ApplyRule::Ptr>& ApplyRule::GetRules(const Type::Ptr& sourceType, const Type::Ptr& targetType)
 {
-	auto it = m_Rules.find(type);
-	if (it == m_Rules.end()) {
-		static std::vector<ApplyRule> emptyList;
-		return emptyList;
+	auto perSourceType (m_Rules.find(sourceType.get()));
+
+	if (perSourceType != m_Rules.end()) {
+		auto perTargetType (perSourceType->second.Regular.find(targetType.get()));
+
+		if (perTargetType != perSourceType->second.Regular.end()) {
+			return perTargetType->second;
+		}
 	}
-	return it->second;
+
+	static const std::vector<ApplyRule::Ptr> noRules;
+	return noRules;
 }
 
 void ApplyRule::CheckMatches(bool silent)
 {
-	for (const RuleMap::value_type& kv : m_Rules) {
-		for (const ApplyRule& rule : kv.second) {
-			if (!rule.HasMatches() && !silent)
-				Log(LogWarning, "ApplyRule")
-					<< "Apply rule '" << rule.GetName() << "' (" << rule.GetDebugInfo() << ") for type '" << kv.first << "' does not match anywhere!";
+	for (auto& perSourceType : m_Rules) {
+		for (auto& perTargetType : perSourceType.second.Regular) {
+			for (auto& rule : perTargetType.second) {
+				CheckMatches(rule, perSourceType.first, silent);
+			}
+		}
+
+		std::unordered_set<ApplyRule*> targeted;
+
+		for (auto& perHost : perSourceType.second.Targeted) {
+			for (auto& rule : perHost.second.ForHost) {
+				targeted.emplace(rule.get());
+			}
+
+			for (auto& perService : perHost.second.ForServices) {
+				for (auto& rule : perService.second) {
+					targeted.emplace(rule.get());
+				}
+			}
+		}
+
+		for (auto rule : targeted) {
+			CheckMatches(rule, perSourceType.first, silent);
 		}
 	}
 }
+
+void ApplyRule::CheckMatches(const ApplyRule::Ptr& rule, Type* sourceType, bool silent)
+{
+	if (!rule->HasMatches() && !silent) {
+		Log(LogWarning, "ApplyRule")
+			<< "Apply rule '" << rule->GetName() << "' (" << rule->GetDebugInfo() << ") for type '"
+			<< sourceType->GetName() << "' does not match anywhere!";
+	}
+}
