@@ -283,6 +283,16 @@ public:
 		: DebuggableExpression(debugInfo), m_Operand1(std::move(operand1)), m_Operand2(std::move(operand2))
 	{ }
 
+	inline const std::unique_ptr<Expression>& GetOperand1() const noexcept
+	{
+		return m_Operand1;
+	}
+
+	inline const std::unique_ptr<Expression>& GetOperand2() const noexcept
+	{
+		return m_Operand2;
+	}
+
 protected:
 	std::unique_ptr<Expression> m_Operand1;
 	std::unique_ptr<Expression> m_Operand2;
@@ -293,7 +303,7 @@ class VariableExpression final : public DebuggableExpression
 public:
 	VariableExpression(String variable, std::vector<Expression::Ptr> imports, const DebugInfo& debugInfo = DebugInfo());
 
-	String GetVariable() const
+	inline const String& GetVariable() const
 	{
 		return m_Variable;
 	}
