@@ -8,37 +8,37 @@ using namespace icinga;
 
 void Checkable::AddDependency(const Dependency::Ptr& dep)
 {
-	boost::mutex::scoped_lock lock(m_DependencyMutex);
+	std::unique_lock<std::mutex> lock(m_DependencyMutex);
 	m_Dependencies.insert(dep);
 }
 
 void Checkable::RemoveDependency(const Dependency::Ptr& dep)
 {
-	boost::mutex::scoped_lock lock(m_DependencyMutex);
+	std::unique_lock<std::mutex> lock(m_DependencyMutex);
 	m_Dependencies.erase(dep);
 }
 
 std::vector<Dependency::Ptr> Checkable::GetDependencies() const
 {
-	boost::mutex::scoped_lock lock(m_DependencyMutex);
+	std::unique_lock<std::mutex> lock(m_DependencyMutex);
 	return std::vector<Dependency::Ptr>(m_Dependencies.begin(), m_Dependencies.end());
 }
 
 void Checkable::AddReverseDependency(const Dependency::Ptr& dep)
 {
-	boost::mutex::scoped_lock lock(m_DependencyMutex);
+	std::unique_lock<std::mutex> lock(m_DependencyMutex);
 	m_ReverseDependencies.insert(dep);
 }
 
 void Checkable::RemoveReverseDependency(const Dependency::Ptr& dep)
 {
-	boost::mutex::scoped_lock lock(m_DependencyMutex);
+	std::unique_lock<std::mutex> lock(m_DependencyMutex);
 	m_ReverseDependencies.erase(dep);
 }
 
 std::vector<Dependency::Ptr> Checkable::GetReverseDependencies() const
 {
-	boost::mutex::scoped_lock lock(m_DependencyMutex);
+	std::unique_lock<std::mutex> lock(m_DependencyMutex);
 	return std::vector<Dependency::Ptr>(m_ReverseDependencies.begin(), m_ReverseDependencies.end());
 }
 
