@@ -9,6 +9,7 @@
 #include "base/utility.hpp"
 #include "base/timer.hpp"
 #include <boost/thread/once.hpp>
+#include <cmath>
 
 using namespace icinga;
 
@@ -22,6 +23,7 @@ boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeAdded;
 boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeRemoved;
 boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeStarted;
 boost::signals2::signal<void (const Downtime::Ptr&)> Downtime::OnDowntimeTriggered;
+boost::signals2::signal<void (const Downtime::Ptr&, const String&, double, const MessageOrigin::Ptr&)> Downtime::OnRemovalInfoChanged;
 
 REGISTER_TYPE(Downtime);
 
@@ -126,11 +128,11 @@ void Downtime::Start(bool runtimeCreated)
 	 * this downtime now *after* it has been added (important
 	 * for DB IDO, etc.)
 	 */
-	if (!checkable->IsStateOK(checkable->GetStateRaw())) {
+	if (!GetFixed() && !checkable->IsStateOK(checkable->GetStateRaw())) {
 		Log(LogNotice, "Downtime")
 			<< "Checkable '" << checkable->GetName() << "' already in a NOT-OK state."
 			<< " Triggering downtime now.";
-		TriggerDowntime();
+		TriggerDowntime(checkable->GetLastStateChange());
 	}
 
 	if (GetFixed() && CanBeTriggered()) {
@@ -138,7 +140,7 @@ void Downtime::Start(bool runtimeCreated)
 		OnDowntimeStarted(this);
 
 		/* Trigger fixed downtime immediately. */
-		TriggerDowntime();
+		TriggerDowntime(std::fmax(GetStartTime(), GetEntryTime()));
 	}
 }
 
@@ -330,7 +332,8 @@ Downtime::Ptr Downtime::AddDowntime(const Checkable::Ptr& checkable, const Strin
 	return downtime;
 }
 
-void Downtime::RemoveDowntime(const String& id, bool includeChildren, bool cancelled, bool expired, const MessageOrigin::Ptr& origin)
+void Downtime::RemoveDowntime(const String& id, bool includeChildren, bool cancelled, bool expired,
+	const String& removedBy, const MessageOrigin::Ptr& origin)
 {
 	Downtime::Ptr downtime = Downtime::GetByName(id);
 
@@ -350,7 +353,9 @@ void Downtime::RemoveDowntime(const String& id, bool includeChildren, bool cance
 		}
 	}
 
-	downtime->SetWasCancelled(cancelled);
+	if (cancelled) {
+		downtime->SetRemovalInfo(removedBy, Utility::GetTime());
+	}
 
 	Array::Ptr errors = new Array();
 
@@ -422,7 +427,7 @@ bool Downtime::CanBeTriggered()
 	return true;
 }
 
-void Downtime::TriggerDowntime()
+void Downtime::TriggerDowntime(double triggerTime)
 {
 	if (!CanBeTriggered())
 		return;
@@ -432,8 +437,9 @@ void Downtime::TriggerDowntime()
 	Log(LogInformation, "Downtime")
 		<< "Triggering downtime '" << GetName() << "' for checkable '" << checkable->GetName() << "'.";
 
-	if (GetTriggerTime() == 0)
-		SetTriggerTime(Utility::GetTime());
+	if (GetTriggerTime() == 0) {
+		SetTriggerTime(triggerTime);
+	}
 
 	Array::Ptr triggers = GetTriggers();
 
@@ -445,13 +451,24 @@ void Downtime::TriggerDowntime()
 			if (!downtime)
 				continue;
 
-			downtime->TriggerDowntime();
+			downtime->TriggerDowntime(triggerTime);
 		}
 	}
 
 	OnDowntimeTriggered(this);
 }
 
+void Downtime::SetRemovalInfo(const String& removedBy, double removeTime, const MessageOrigin::Ptr& origin) {
+	{
+		ObjectLock olock(this);
+
+		SetRemovedBy(removedBy, false, origin);
+		SetRemoveTime(removeTime, false, origin);
+	}
+
+	OnRemovalInfoChanged(this, removedBy, removeTime, origin);
+}
+
 String Downtime::GetDowntimeIDFromLegacyID(int id)
 {
 	std::unique_lock<std::mutex> lock(l_DowntimeMutex);
@@ -475,7 +492,7 @@ void Downtime::DowntimesStartTimerHandler()
 			OnDowntimeStarted(downtime);
 
 			/* Trigger fixed downtime immediately. */
-			downtime->TriggerDowntime();
+			downtime->TriggerDowntime(std::fmax(downtime->GetStartTime(), downtime->GetEntryTime()));
 		}
 	}
 }
