@@ -25,16 +25,14 @@ using namespace icinga;
 using Prio = RedisConnection::QueryPriority;
 
 String IcingaDB::m_EnvironmentId;
-std::once_flag IcingaDB::m_EnvironmentIdOnce;
+std::mutex IcingaDB::m_EnvironmentIdInitMutex;
 
 REGISTER_TYPE(IcingaDB);
 
-REGISTER_STATSFUNCTION(IcingaDB, &IcingaDB::StatsFunc);
-
 IcingaDB::IcingaDB()
 	: m_Rcon(nullptr)
 {
-	m_Rcon = nullptr;
+	m_RconLocked.store(nullptr);
 
 	m_WorkQueue.SetName("IcingaDB");
 
@@ -42,28 +40,6 @@ IcingaDB::IcingaDB()
 	m_PrefixConfigCheckSum = "icinga:checksum:";
 }
 
-/**
- * Feature stats interface
- *
- * @param status Key value pairs for feature stats
- */
-void IcingaDB::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr& perfdata)
-{
-	DictionaryData nodes;
-
-	for (auto& icingadb : ConfigType::GetObjectsByType<IcingaDB>()) {
-		auto historyBufferItems (icingadb->m_HistoryBulker.Size());
-
-		nodes.emplace_back(icingadb->GetName(), new Dictionary({
-			{ "history_buffer_items", historyBufferItems }
-		}));
-
-		perfdata->Add(new PerfdataValue("icingadb_" + icingadb->GetName() + "_history_buffer_items", historyBufferItems));
-	}
-
-	status->Set("icingadb", new Dictionary(std::move(nodes)));
-}
-
 void IcingaDB::Validate(int types, const ValidationUtils& utils)
 {
 	ObjectImpl<IcingaDB>::Validate(types, utils);
@@ -74,6 +50,13 @@ void IcingaDB::Validate(int types, const ValidationUtils& utils)
 	if (GetEnableTls() && GetCertPath().IsEmpty() != GetKeyPath().IsEmpty()) {
 		BOOST_THROW_EXCEPTION(ValidationError(this, std::vector<String>(), "Validation failed: Either both a client certificate (cert_path) and its private key (key_path) or none of them must be given."));
 	}
+
+	try {
+		InitEnvironmentId();
+	} catch (const std::exception& e) {
+		BOOST_THROW_EXCEPTION(ValidationError(this, std::vector<String>(),
+			String("Validation failed: ") + e.what()));
+	}
 }
 
 /**
@@ -83,39 +66,8 @@ void IcingaDB::Start(bool runtimeCreated)
 {
 	ObjectImpl<IcingaDB>::Start(runtimeCreated);
 
-	std::call_once(m_EnvironmentIdOnce, []() {
-		String path = Configuration::DataDir + "/icingadb.env";
-
-		if (Utility::PathExists(path)) {
-			m_EnvironmentId = Utility::LoadJsonFile(path);
-
-			if (m_EnvironmentId.GetLength() != 2*SHA_DIGEST_LENGTH) {
-				throw std::runtime_error("Wrong length of stored Icinga DB environment");
-			}
-
-			for (unsigned char c : m_EnvironmentId) {
-				if (!std::isxdigit(c)) {
-					throw std::runtime_error("Stored Icinga DB environment is not a hex string");
-				}
-			}
-		} else {
-			std::shared_ptr<X509> cert = GetX509Certificate(ApiListener::GetDefaultCaPath());
-
-			unsigned int n;
-			unsigned char digest[EVP_MAX_MD_SIZE];
-			if (X509_pubkey_digest(cert.get(), EVP_sha1(), digest, &n) != 1) {
-				BOOST_THROW_EXCEPTION(openssl_error()
-					<< boost::errinfo_api_function("X509_pubkey_digest")
-					<< errinfo_openssl_error(ERR_peek_error()));
-			}
-
-			m_EnvironmentId = BinaryToHex(digest, n);
-
-			Utility::SaveJsonFile(path, 0600, m_EnvironmentId);
-		}
-
-		m_EnvironmentId = m_EnvironmentId.ToLower();
-	});
+	VERIFY(!m_EnvironmentId.IsEmpty());
+	PersistEnvironmentId();
 
 	Log(LogInformation, "IcingaDB")
 		<< "'" << GetName() << "' started.";
@@ -128,6 +80,7 @@ void IcingaDB::Start(bool runtimeCreated)
 	m_Rcon = new RedisConnection(GetHost(), GetPort(), GetPath(), GetPassword(), GetDbIndex(),
 		GetEnableTls(), GetInsecureNoverify(), GetCertPath(), GetKeyPath(), GetCaPath(), GetCrlPath(),
 		GetTlsProtocolmin(), GetCipherList(), GetConnectTimeout(), GetDebugInfo());
+	m_RconLocked.store(m_Rcon);
 
 	for (const Type::Ptr& type : GetTypes()) {
 		auto ctype (dynamic_cast<ConfigType*>(type.get()));
@@ -288,3 +241,69 @@ bool IcingaDB::DumpedGlobals::IsNew(const String& id)
 	std::lock_guard<std::mutex> l (m_Mutex);
 	return m_Ids.emplace(id).second;
 }
+
+/**
+ * Initializes the m_EnvironmentId attribute or throws an exception on failure to do so. Can be called concurrently.
+ */
+void IcingaDB::InitEnvironmentId()
+{
+	// Initialize m_EnvironmentId once across all IcingaDB objects. In theory, this could be done using
+	// std::call_once, however, due to a bug in libstdc++ (https://gcc.gnu.org/bugzilla/show_bug.cgi?id=66146),
+	// this can result in a deadlock when an exception is thrown (which is explicitly allowed by the standard).
+	std::unique_lock<std::mutex> lock (m_EnvironmentIdInitMutex);
+
+	if (m_EnvironmentId.IsEmpty()) {
+		String path = Configuration::DataDir + "/icingadb.env";
+		String envId;
+
+		if (Utility::PathExists(path)) {
+			envId = Utility::LoadJsonFile(path);
+
+			if (envId.GetLength() != 2*SHA_DIGEST_LENGTH) {
+				throw std::runtime_error("environment ID stored at " + path + " is corrupt: wrong length.");
+			}
+
+			for (unsigned char c : envId) {
+				if (!std::isxdigit(c)) {
+					throw std::runtime_error("environment ID stored at " + path + " is corrupt: invalid hex string.");
+				}
+			}
+		} else {
+			String caPath = ApiListener::GetDefaultCaPath();
+
+			if (!Utility::PathExists(caPath)) {
+				throw std::runtime_error("Cannot find the CA certificate at '" + caPath + "'. "
+					"Please ensure the ApiListener is enabled first using 'icinga2 api setup'.");
+			}
+
+			std::shared_ptr<X509> cert = GetX509Certificate(caPath);
+
+			unsigned int n;
+			unsigned char digest[EVP_MAX_MD_SIZE];
+			if (X509_pubkey_digest(cert.get(), EVP_sha1(), digest, &n) != 1) {
+				BOOST_THROW_EXCEPTION(openssl_error()
+											  << boost::errinfo_api_function("X509_pubkey_digest")
+											  << errinfo_openssl_error(ERR_peek_error()));
+			}
+
+			envId = BinaryToHex(digest, n);
+		}
+
+		m_EnvironmentId = envId.ToLower();
+	}
+}
+
+/**
+ * Ensures that the environment ID is persisted on disk or throws an exception on failure to do so.
+ * Can be called concurrently.
+ */
+void IcingaDB::PersistEnvironmentId()
+{
+	String path = Configuration::DataDir + "/icingadb.env";
+
+	std::unique_lock<std::mutex> lock (m_EnvironmentIdInitMutex);
+
+	if (!Utility::PathExists(path)) {
+		Utility::SaveJsonFile(path, 0600, m_EnvironmentId);
+	}
+}
