@@ -74,6 +74,16 @@ namespace icinga
 			SyncConnection = 255
 		};
 
+		struct QueryAffects
+		{
+			size_t Config;
+			size_t State;
+			size_t History;
+
+			QueryAffects(size_t config = 0, size_t state = 0, size_t history = 0)
+				: Config(config), State(state), History(history) { }
+		};
+
 		RedisConnection(const String& host, int port, const String& path, const String& password, int db,
 			bool useTls, bool insecure, const String& certPath, const String& keyPath, const String& caPath, const String& crlPath,
 			const String& tlsProtocolmin, const String& cipherList, double connectTimeout, DebugInfo di, const Ptr& parent = nullptr);
@@ -84,20 +94,48 @@ namespace icinga
 
 		bool IsConnected();
 
-		void FireAndForgetQuery(Query query, QueryPriority priority);
-		void FireAndForgetQueries(Queries queries, QueryPriority priority);
+		void FireAndForgetQuery(Query query, QueryPriority priority, QueryAffects affects = {});
+		void FireAndForgetQueries(Queries queries, QueryPriority priority, QueryAffects affects = {});
 
-		Reply GetResultOfQuery(Query query, QueryPriority priority);
-		Replies GetResultsOfQueries(Queries queries, QueryPriority priority);
+		Reply GetResultOfQuery(Query query, QueryPriority priority, QueryAffects affects = {});
+		Replies GetResultsOfQueries(Queries queries, QueryPriority priority, QueryAffects affects = {});
 
 		void EnqueueCallback(const std::function<void(boost::asio::yield_context&)>& callback, QueryPriority priority);
 		void Sync();
+		double GetOldestPendingQueryTs();
 
 		void SuppressQueryKind(QueryPriority kind);
 		void UnsuppressQueryKind(QueryPriority kind);
 
 		void SetConnectedCallback(std::function<void(boost::asio::yield_context& yc)> callback);
 
+		inline bool GetConnected()
+		{
+			return m_Connected.load();
+		}
+
+		int GetQueryCount(RingBuffer::SizeType span);
+
+		inline int GetPendingQueryCount()
+		{
+			return m_PendingQueries;
+		}
+
+		inline int GetWrittenConfigFor(RingBuffer::SizeType span, RingBuffer::SizeType tv = Utility::GetTime())
+		{
+			return m_WrittenConfig.UpdateAndGetValues(tv, span);
+		}
+
+		inline int GetWrittenStateFor(RingBuffer::SizeType span, RingBuffer::SizeType tv = Utility::GetTime())
+		{
+			return m_WrittenState.UpdateAndGetValues(tv, span);
+		}
+
+		inline int GetWrittenHistoryFor(RingBuffer::SizeType span, RingBuffer::SizeType tv = Utility::GetTime())
+		{
+			return m_WrittenHistory.UpdateAndGetValues(tv, span);
+		}
+
 	private:
 		/**
 		 * What to do with the responses to Redis queries.
@@ -134,6 +172,9 @@ namespace icinga
 			Shared<std::pair<Query, std::promise<Reply>>>::Ptr GetResultOfQuery;
 			Shared<std::pair<Queries, std::promise<Replies>>>::Ptr GetResultsOfQueries;
 			std::function<void(boost::asio::yield_context&)> Callback;
+
+			double CTime;
+			QueryAffects Affects;
 		};
 
 		typedef boost::asio::ip::tcp Tcp;
@@ -175,6 +216,7 @@ namespace icinga
 
 		void IncreasePendingQueries(int count);
 		void DecreasePendingQueries(int count);
+		void RecordAffected(QueryAffects affected, double when);
 
 		template<class StreamPtr>
 		void Handshake(StreamPtr& stream, boost::asio::yield_context& yc);
@@ -225,7 +267,10 @@ namespace icinga
 
 		// Stats
 		RingBuffer m_InputQueries{10};
-		RingBuffer m_OutputQueries{10};
+		RingBuffer m_OutputQueries{15 * 60};
+		RingBuffer m_WrittenConfig{15 * 60};
+		RingBuffer m_WrittenState{15 * 60};
+		RingBuffer m_WrittenHistory{15 * 60};
 		int m_PendingQueries{0};
 		boost::asio::deadline_timer m_LogStatsTimer;
 		Ptr m_Parent;
@@ -555,12 +600,12 @@ Value RedisConnection::ReadRESP(AsyncReadStream& stream, boost::asio::yield_cont
 					throw BadRedisInt(std::move(buf));
 				}
 
-				Array::Ptr arr = new Array();
-
 				if (i < 0) {
-					i = 0;
+					return Empty;
 				}
 
+				Array::Ptr arr = new Array();
+
 				arr->Reserve(i);
 
 				for (; i; --i) {
