@@ -246,7 +246,7 @@ bool HandleAccessControl(
 			if (!allowedOrigins.empty()) {
 				auto& origin (request[http::field::origin]);
 
-				if (allowedOrigins.find(std::string(origin)) != allowedOrigins.end()) {
+				if (allowedOrigins.find(origin.to_string()) != allowedOrigins.end()) {
 					response.set(http::field::access_control_allow_origin, origin);
 				}
 
@@ -257,7 +257,7 @@ bool HandleAccessControl(
 				if (request.method() == http::verb::options && !request[http::field::access_control_request_method].empty()) {
 					response.result(http::status::ok);
 					response.set(http::field::access_control_allow_methods, "GET, POST, PUT, DELETE");
-					response.set(http::field::access_control_allow_headers, "Authorization, X-HTTP-Method-Override");
+					response.set(http::field::access_control_allow_headers, "Authorization, Content-Type, X-HTTP-Method-Override");
 					response.body() = "Preflight OK";
 					response.content_length(response.body().size());
 					response.set(http::field::connection, "close");
@@ -536,7 +536,7 @@ void HttpServerConnection::ProcessMessages(boost::asio::yield_context yc)
 			if (!authenticatedUser) {
 				CpuBoundWork fetchingAuthenticatedUser (yc);
 
-				authenticatedUser = ApiUser::GetByAuthHeader(std::string(request[http::field::authorization]));
+				authenticatedUser = ApiUser::GetByAuthHeader(request[http::field::authorization].to_string());
 			}
 
 			Log logMsg (LogInformation, "HttpServerConnection");
