@@ -42,7 +42,7 @@ class JsonRpcConnection final : public Object
 public:
 	DECLARE_PTR_TYPEDEFS(JsonRpcConnection);
 
-	JsonRpcConnection(const String& identity, bool authenticated, const std::shared_ptr<AsioTlsStream>& stream, ConnectionRole role);
+	JsonRpcConnection(const String& identity, bool authenticated, const Shared<AsioTlsStream>::Ptr& stream, ConnectionRole role);
 
 	void Start();
 
@@ -50,7 +50,7 @@ public:
 	String GetIdentity() const;
 	bool IsAuthenticated() const;
 	Endpoint::Ptr GetEndpoint() const;
-	std::shared_ptr<AsioTlsStream> GetStream() const;
+	Shared<AsioTlsStream>::Ptr GetStream() const;
 	ConnectionRole GetRole() const;
 
 	void Disconnect();
@@ -68,7 +68,7 @@ private:
 	String m_Identity;
 	bool m_Authenticated;
 	Endpoint::Ptr m_Endpoint;
-	std::shared_ptr<AsioTlsStream> m_Stream;
+	Shared<AsioTlsStream>::Ptr m_Stream;
 	ConnectionRole m_Role;
 	double m_Timestamp;
 	double m_Seen;
@@ -80,7 +80,7 @@ private:
 	bool m_ShuttingDown;
 	boost::asio::deadline_timer m_CheckLivenessTimer, m_HeartbeatTimer;
 
-	JsonRpcConnection(const String& identity, bool authenticated, const std::shared_ptr<AsioTlsStream>& stream, ConnectionRole role, boost::asio::io_context& io);
+	JsonRpcConnection(const String& identity, bool authenticated, const Shared<AsioTlsStream>::Ptr& stream, ConnectionRole role, boost::asio::io_context& io);
 
 	void HandleIncomingMessages(boost::asio::yield_context yc);
 	void WriteOutgoingMessages(boost::asio::yield_context yc);
